#include <cstdio>
#include <iostream>
#include <string>
#include <argh.hh>

using namespace std;

typedef unsigned long ip;

static ip makebroadcast(ip addr, ip mask)
{
    return addr | ~mask;
}

/* Picks up n.n.n.n from parameter */
static ip getip(const string &str)
{
    const char *const s = str.c_str();
    ip result = 0;
    unsigned p=0;
    for(;;)
    {
        unsigned nip = 0;
        while(s[p]>='0' && s[p]<='9')
            nip = nip*10 + (s[p++]-'0');
        result = result*256 + nip;
        if(s[p] != '.')break;
        ++p;
    }
    return result;
}

/* Returns the network begin address */
static ip unmask(ip addr, ip mask)
{
    return addr & mask;
}

/* Picks up /n from n.n.n.n/n and returns the netmask */
static ip getmask(const string &str)
{
    const char *q;
    const char *const s = str.c_str();
    for(q=s; *q && *q!='/'; ++q);
    if(!*q)
    {
        for(q=s; *q && *q!='.'; ++q);
        if(*q=='.')return ip(~0);
    }
    unsigned p = *q ? (q-s)+1 : 0;
    
    unsigned width = 0;
    while(s[p]>='0' && s[p]<='9')
        width = width*10 + (s[p++] - '0');

    if(!width)return 0;
    if(width > 32)width = 32;
    
    return ~(ip(1) << (32-width)) + 1;
}

static unsigned nullbits(ip mask)
{
    unsigned b=0;
    while(!(mask & (1<<b)) && b<32)++b;
    return b;
}

static string IP2str(const ip adr)
{
    char p[16]; /* 255.255.255.255 = 15 chars, plus nul */
    if(adr < 256)
        sprintf(p, "%u", static_cast<unsigned> (adr));
    else
        sprintf(p, "%u.%u.%u.%u",
            static_cast<unsigned> ((adr>>24) & 255),
            static_cast<unsigned> ((adr>>16) & 255),
            static_cast<unsigned> ((adr>> 8) & 255),
            static_cast<unsigned> ((adr    ) & 255));
    return p;
}

class interface
{
    ip begin, addr, mask, broad;
    unsigned width;
    void make_broad_begin_width()
    {
        broad = makebroadcast(addr, mask);
        begin = unmask(addr, mask);
        width = 32-nullbits(mask);
    }
public:
    interface()
    {
    }
    
    void init_addr_bits(const string &s)
    {
        addr = getip(s);
        mask = getmask(s);
        
        make_broad_begin_width();
    }
    void init(const string &s_addr, const string &s_mask)
    {
        addr = getip(s_addr);
        mask = getip(s_mask);
        
        make_broad_begin_width();
    }
    void dump(bool b_begin, bool b_addr, bool b_wid, bool b_mask, bool b_broad)
    {
        bool c = false;
        if(b_begin) { if(!c)c=true;else cout << '\t'; cout << IP2str(begin); }
        if(b_addr)  { if(!c)c=true;else cout << '\t'; cout << IP2str(mask); }
        if(b_wid)   { if(!c)c=true;else cout << '/'; cout << width; }
        if(b_mask)  { if(!c)c=true;else cout << '\t'; cout << IP2str(addr); }
        if(b_broad) { if(!c)c=true;else cout << '\t'; cout << IP2str(broad); }
        cout << endl;
    }
};

static bool IsProg(const char *a0, const char *const name)
{
p1: for(const char *s = a0; *s; ++s)
        if(*s=='/') {a0=s+1; goto p1; }
    while(*a0 && *name)
        if(*a0 != *name)return false;
    return *a0 == *name;
}

int main(int argc, const char *const *argv)
{
    bool begin=false, addr=true, width=true, mask=true, broad=true;
    interface i;
    
    if(IsProg(argv[0], "getipmask"))
    {
        begin=true; addr=false; mask=true; broad=false; width=false;
    }
    
    ParamHandler Argh;

    Argh.AddLong("help",    'h').SetBool().SetDesc("This help.");
    Argh.AddLong("version", 'V').SetBool().SetDesc("Displays version information.");
    
    Argh.StartParse(argc, argv);
    
    string mysteeri;
    
    for(;;)
    {
        int c = Argh.GetParam();
        if(c == -1)break;
        switch(c)
        {
            case 'V': cout << VERSION << endl;
                return 0;
            case 'h':
                cout <<
                    "iputils "VERSION" - Copyright (C) 1992,2002 Bisqwit (http://iki.fi/bisqwit/)\n"
                    "\n"
                    "\nUsage: software [<option> [<...>]]\n"
                    "\nOptions:\n";
                Argh.ListOptions();
                cout << "\nNo warranty whatsoever.\n";
                return 0;
            default:
                mysteeri = Argh.GetString();
        }
    }
    if(!Argh.ok())return -1;
    
    i.dump(begin, addr, width, mask, broad);
    
    return 0;
}
